# MXM Reporter 0.4b (2009-06-16)
# by: Vodka Martini

require 'sketchup.rb'


unless file_loaded?('mxm_reporter.rb')
  UI.menu("Plugins").add_item('MXM Reporter') { RMG::MXM_REPORTER.create }
  file_loaded('mxm_reporter.rb')
end



module RMG

  def RMG.windows_platform?()
    return ENV['OS'] =~ /windows/i
  end
  def RMG.escapeHTML(str)
    str.gsub(/&/n, '&amp;').gsub(/\"/n, '&quot;').gsub(/>/n, '&gt;').gsub(/</n, '&lt;')
  end
  def RMG.encode64(bin)
    [bin].pack("m").gsub(/[\n]/, "_")
  end
  def RMG.decode64(str)
    str.gsub(/_/, "\n").unpack("m")[0]
  end
  
  module MXM_REPORTER
   
    @reporter = nil
    @materials_observer = nil
    
    def MXM_REPORTER.reporter()
      return @reporter
    end
    
    def MXM_REPORTER.create()
      if (@reporter != nil)
        @reporter.close
      end
      if (@materials_observer == nil)
        @materials_observer = MXMReportMaterialsObserver.new
      end
      
      Sketchup.active_model.materials.add_observer(@materials_observer)
      @reporter = MXMReportDialog.new
      
      # manually closing the webdialog doesn't invoke the WebDialog.close() method, 
      # but it WILL invoke the set_on_close block.
      @reporter.set_on_close do
        Sketchup.active_model.materials.remove_observer(@materials_observer)
        @reporter = nil
      end
      return @reporter
    end
    
    class MXMReportMaterialsObserver < Sketchup::MaterialsObserver
      def onMaterialSetCurrent(materials, material)
        if (material == nil)
          js = "select_material('')"
        else
          js = "select_material('#{RMG.encode64(material.name)}')"
        end
        if (RMG::MXM_REPORTER.reporter != nil)
          RMG::MXM_REPORTER.reporter.execute_script(js);
        end
      end
      def onMaterialAdd(materials, material)
        if (RMG::MXM_REPORTER.reporter != nil)
          RMG::MXM_REPORTER.reporter.add_material(material)
        end
      end      
      def onMaterialRemove(materials, material)
        if (RMG::MXM_REPORTER.reporter != nil)
          RMG::MXM_REPORTER.reporter.remove_material(material)
        end
      end
    end
    
    class MXMReportDialog < UI::WebDialog
      
      attr_accessor :auto_mxm_enabled
      attr_accessor :auto_mxm_path
      attr_accessor :auto_mxm_materials
      attr_accessor :recent_mxm_directory
      
      
      def initialize()
        super("MXM Reporter", true, "MXM Reporter", 640, 480, 100, 100, true)
        self.min_width = 640
        self.min_height = 480
        self.navigation_buttons_enabled = false
        self.add_action_callback("onclick_sketchup_material") do |dialog, params|
          material_name = RMG.decode64(params)
          select_material(material_name)
        end
        self.add_action_callback("onclick_mxm_file") do |dialog, params|
          filename = RMG.decode64(params)
          edit_mxm(filename)
        end
        self.add_action_callback("onclick_unassigned_mxm_file") do |dialog, params|
          material_name = RMG.decode64(params)
          assign_mxm(material_name)
        end
        self.add_action_callback("onclick_assigned_mxm_file") do |dialog, params|
          args = params.split('&')
          material_name = RMG.decode64(args[0])
          if (args[1] != nil)
            location = RMG.decode64(args[1])
          else
            location = nil
          end
          reassign_mxm(material_name, location)
        end
        self.add_action_callback("onclick_paste_mxm_location") do |dialog, params|
          args = params.split('&')
          material_name = RMG.decode64(args[0])
          location = RMG.decode64(args[1])
          paste_mxm(material_name, location)
        end
        self.add_action_callback("onchange_auto_mxm_toggle") do
          toggle_auto_mxm()
        end
        self.add_action_callback("onclick_scan_auto_mxm_path") do
          scan_auto_mxm_path()
        end
        self.add_action_callback("onclick_remove_mxm_dictionary") do
          return if (Sketchup.active_model == nil)
          remove_mxm_dictionary(Sketchup.active_model.materials.current)
        end
        self.add_action_callback("onclick_remove_all_mxm_dictionaries") do
          remove_all_mxm_dictionaries()
        end
        self.add_action_callback("onclick_refresh") do
          refresh()
        end
        
        init_auto_mxm_settings()
        @recent_mxm_directory = nil
        self.refresh()
      end #initialize
     
      def init_auto_mxm_settings()
        if (@auto_mxm_path == nil)
          @auto_mxm_path = "c:\\materials"
          if (File.exist?("C:\\Windows\\MaxwellRender.ini"))
            File.open("C:\\Windows\\MaxwellRender.ini", "r") do |file|
              entries = file.readlines().select {|line| line =~ /^MXMPath/}
              if (entries != nil)
                path = entries.first.gsub(/^MXMPath=/, "").strip
                @auto_mxm_path = path
              end
            end
          end
          home = ENV['HOME']
          if (home != nil)
            filename = home + "/MaxwellRender.ini"
            if (File.exist?(filename))
              File.open(filename, "r") do |file|
                entries = file.readlines().select {|line| line =~ /^MXMPath/}
                if (entries != nil)
                  path = entries.first.gsub(/^MXMPath=/, "").strip
                  @auto_mxm_path = path
                end
              end
            end
          end
        end
        if (@auto_mxm_enabled == nil)
          @auto_mxm_enabled = false
        end
        if (@auto_mxm_materials == nil)
          @auto_mxm_materials = {}
        end
      end
      
      # called from webdialog js when assigned mxm slot is selected
      def edit_mxm(filename)
        if (RMG.windows_platform?)
          maxwell_root = ENV['MAXWELL_ROOT']
          if (maxwell_root == nil)
            maxwell_root = 'c:\Program Files\Next Limit\Maxwell'
          end
          mxed_path = maxwell_root + '\mxed.exe'
        else #OSX
          mxed_path = '/Applications/Maxwell/mxed.app/Contents/MacOS/mxed'
        end
        
        if (File.exists?(mxed_path)) && (File.exists?(filename))
          if (RMG.windows_platform?)
            cmd = 'start /B /D"' + maxwell_root + '" mxed.exe -mxm:"' + filename + '"'
          else
            cmd = mxed_path + ' -mxm:"' + filename + '"'
          end
          #puts cmd
          system(cmd)
        end
      end
      
      # Called from webdialog js when unassigned mxm slot is selected
      def assign_mxm(material_name)
        materials = Sketchup.active_model.materials  
        return if ((materials == nil) || (material_name == nil))
        material = materials["#{material_name}"]
        if (material != nil)
          recent_mxm_directory = @recent_mxm_directory
          if (recent_mxm_directory == nil)
            recent_mxm_directory = get_element_value("auto_mxm_path").gsub(/\\/, "/").gsub(/[\/]+$/, "")
          end
          if (File.directory?(recent_mxm_directory))
            filename = UI.openpanel("Choose MXM", recent_mxm_directory, "*.mxm")
          else
            filename = UI.openpanel("Choose MXM", "", "*.mxm")
          end
          if (filename != nil) && (File.exist?(filename))
            @recent_mxm_directory = File.dirname(filename)
            material.attribute_dictionary("MaxwellMXM", true)
            material.set_attribute("MaxwellMXM", "Material_Type", 0)
            material.set_attribute("MaxwellMXM", "MXM", filename)
            material.set_attribute("MaxwellMXM", "MXMEnabled", 1)
            update_material(material)
          end
        end
      end
      def reassign_mxm(material_name, current_filename)
        materials = Sketchup.active_model.materials  
        return if ((materials == nil) || (material_name == nil))
        material = materials["#{material_name}"]
        if (material != nil)
          recent_mxm_directory = @recent_mxm_directory
          if (recent_mxm_directory == nil)
            recent_mxm_directory = get_element_value("auto_mxm_path").gsub(/\\/, "/").gsub(/[\/]+$/, "")
          end        
          if (current_filename != nil) && (File.directory?(File.dirname(current_filename)))
            filename = UI.openpanel("Choose MXM", File.dirname(current_filename), "*.mxm")
          elsif (File.directory?(recent_mxm_directory))
            filename = UI.openpanel("Choose MXM", recent_mxm_directory, "*.mxm")
          else
            filename = UI.openpanel("Choose MXM", "", "*.mxm")
          end
          if (filename != nil) && (File.exist?(filename))
            @recent_mxm_directory = File.dirname(filename)
            material.attribute_dictionary("MaxwellMXM", true)
            material.set_attribute("MaxwellMXM", "Material_Type", 0)
            material.set_attribute("MaxwellMXM", "MXM", filename)
            material.set_attribute("MaxwellMXM", "MXMEnabled", 1)
            update_material(material)
          end
        end
      end
      def paste_mxm(material_name, filename)
        materials = Sketchup.active_model.materials 
        return if ((materials == nil) || (material_name == nil))
        material = materials["#{material_name}"]
        if (material != nil)
          material.attribute_dictionary("MaxwellMXM", true)
          material.set_attribute("MaxwellMXM", "Material_Type", 0)
          material.set_attribute("MaxwellMXM", "MXM", filename)
          material.set_attribute("MaxwellMXM", "MXMEnabled", 1)
          update_material(material)
        end
      end
      
      # Called from webdialog js when sketchup material name is selected.
      # Sets current material, which fires off observer, which executes js to highlight
      def select_material(material_name)
        materials = Sketchup.active_model.materials  
        if (materials == nil) || (material_name == nil)
          return
        end
        if (materials["#{material_name}"] != nil)
          materials.current = materials["#{material_name}"]
        end
      end
      
      # Called from observer
      def add_material(material)
        id = 'tr-' + RMG.encode64(material.name)      
        html = sketchup_material_info(material) + maxwell_material_info(material)
        html.gsub!(/\\/, "\\\\\\")      #       \ -> \\
        html.gsub!(/'/, "\\\\'")        #       ' -> \'
        html.gsub!(/\n/, "\\\\n")       # newline -> \n
        js = "add_material('#{id}', '#{html}')"
        execute_script(js);
      end
      
      # Called from ruby when changing mxm info on an individual material
      # Updates appropriate table row without having to refresh entire page
      def update_material(material)
        if (material != nil)
          id = 'tr-' + RMG.encode64(material.name)
          html = sketchup_material_info(material) + maxwell_material_info(material)
          html.gsub!(/\\/, "\\\\\\")      #       \ -> \\
          html.gsub!(/'/, "\\\\'")        #       ' -> \'
          html.gsub!(/\n/, "\\\\n")       # newline -> \n
          js = "update_material('#{id}', '#{html}')"
          execute_script(js);
        end
      end
      
      # Called from observer
      def remove_material(material)
        js = "remove_material('#{RMG.encode64(material.name)}')"
        execute_script(js);
      end
      
      # Called from webdialog js when auto_mxm_toggle is changed
      def toggle_auto_mxm()
        setting = get_element_value("auto_mxm_toggle")
        if (setting == "enabled")
          @auto_mxm_enabled = true
        else
          @auto_mxm_enabled = false
        end
        if (!@auto_mxm_materials.empty?)
          refresh()
        end
      end
      
      def remove_mxm_dictionary(material)
        return if (material == nil)
        mxm = material.attribute_dictionary('MaxwellMXM')      
        if (mxm != nil) 
          material.attribute_dictionaries.delete('MaxwellMXM')
          update_material(material)
        end
      end
      
      def remove_all_mxm_dictionaries()
        return if (Sketchup.active_model == nil)
        materials = Sketchup.active_model.materials
        materials.each do |material|
          remove_mxm_dictionary(material)
        end
      end
      
      
      def refresh()
        return if (Sketchup.active_model == nil)
        materials = Sketchup.active_model.materials.sort_by {|m| m.display_name.downcase}
        report = ""
        materials.each do |material|
          report << "<tr id=\"#{'tr-'+RMG.encode64(material.name)}\">\n"
          report << "    " << sketchup_material_info(material)
          report << "    " << maxwell_material_info(material)
          report << "</tr>\n"
        end
        
        html = HTML_REPORT.gsub(/INSERT_REPORT/, report)
        html.gsub!(/INSERT_AUTOMXM_LOCATION/, @auto_mxm_path)
        if (@auto_mxm_enabled)
          html.gsub!(/INSERT_AUTOMXM_SELECTED/, ' selected="selected"')
          html.gsub!(/INSERT_AUTOMXM_PATH_DISABLED/, '')
          html.gsub!(/INSERT_SCAN_BUTTON_DISABLED/, '')
        else
          html.gsub!(/INSERT_AUTOMXM_SELECTED/, '')
          html.gsub!(/INSERT_AUTOMXM_PATH_DISABLED/, ' disabled="disabled"')
          html.gsub!(/INSERT_SCAN_BUTTON_DISABLED/, ' disabled="disabled"')
        end
        self.set_html(html)
        self.show()
      end
      
      def scan_auto_mxm_path()
        dir = get_element_value("auto_mxm_path").gsub(/\\/, "/").gsub(/[\/]+$/, "")
        @auto_mxm_path = dir # if File.directory?(dir)
        @auto_mxm_materials = {}
        summary = "<h1>Auto MXM Scan Results</h1>\n"
        summary << "<h2>#{RMG::escapeHTML(dir)}</h2><br />\n"
        scan_file(dir, summary)
        count = @auto_mxm_materials.length
        ambiguous = ""
        @auto_mxm_materials.each_value do |material|
          if (material.length > 1)
            material.each_index do |i|
              ambiguous << "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\n"
              if (i == 0)
                ambiguous << "#{RMG::escapeHTML(material[i])}<br />\n"
              else
                ambiguous << '<span class="repeat">'
                ambiguous << "#{RMG::escapeHTML(material[i])}</span><br />\n"
              end
            end
            ambiguous << "<br />\n"
          end
        end
        if (ambiguous.length > 0)
          summary << "<br /><br /><h3>Ambiguous Material Names:</h3><br />\n"
          summary << ambiguous
        end
        summary << "<br /><hr />\n"
        summary << "<br /><strong>Located #{count} Materials</strong><br /><br />\n"
        html = HTML_SUMMARY.gsub(/insert_summary/, summary)
        self.set_html(html)
        self.show()
      end
        
      def scan_file(filename, summary)
        if (filename =~ /\.mxm/) && File.file?(filename)
          basename = File.basename(filename, ".mxm")
          material_name = RMG::encode64(basename.downcase)
          if (@auto_mxm_materials["#{material_name}"] == nil)
            @auto_mxm_materials["#{material_name}"] = [filename]
            summary << "#{RMG::escapeHTML(filename)}<br />\n"
          else
            @auto_mxm_materials["#{material_name}"] << filename
            summary << "<span class=\"repeat\">#{RMG::escapeHTML(filename)}</span><br />\n"
          end
        elsif File.directory?(filename)
          Dir[filename + "/*"].each { |f| scan_file(f, summary) }
        end
      end
      
      private
      
      def context_menu(material, filename)
        if (filename != nil) && (File.exist?(filename))
          params = "'#{RMG.encode64(material.name)}', '#{RMG.encode64(filename)}'"
        else
          params = "'#{RMG.encode64(material.name)}'"
        end
        return "oncontextmenu=\"show_context_menu(event, #{params}); return false;\""
      end
      
      def sketchup_material_info(material)
        selected = ''
        if (Sketchup.active_model.materials.current == material)
          selected = ' selected'
        end 
        info = ""
        info << '<td id="' << RMG.encode64(material.name) << '"' 
        info << ' class="selectable' << selected << '"'
        info << " onclick=\"sketchup_material_onClick('#{RMG.encode64(material.name)}')\">"
        info << RMG.escapeHTML(material.display_name) << "</td>\n"
        return info
      end
      
      def maxwell_material_info(material)
        info = ""
        mxm = material.attribute_dictionary('MaxwellMXM')      
        if (mxm == nil)
          if (@auto_mxm_enabled)
            info << maxwell_auto_mxm_material_info(material)
          else
            info << "<td #{context_menu(material, nil)}><span class=\"mxm_file\""
            info << " onclick=\"unassigned_mxm_file_onClick('#{RMG.encode64(material.name)}')\">"
            info << "no material assigned</span></td>\n"
          end
        elsif (mxm['Material_Type'] == 0)
          info << maxwell_mxm_material_info(material)
        elsif (mxm['Material_Type'] == 1)
          info << "<td class=\"unknown\" #{context_menu(material, nil)}>Material Type: 1</td>\n"
        elsif (mxm['Material_Type'] == 2)
          info << maxwell_plastic_material_info(material)
        elsif (mxm['Material_Type'] == 3)
          info << maxwell_metal_material_info(material)
        elsif (mxm['Material_Type'] == 4)
          info << "<td class=\"unknown\" #{context_menu(material, nil)}>Material_Type: 4</td>\n"
        elsif (mxm['Material_Type'] == 5)
          info << maxwell_emitter_material_info(material)
        else
          info << "<td class=\"unknown\" #{context_menu(material, nil)}>"
          info << "#{RMG.escapeHTML(mxm.to_a.inspect)}</td>\n"
        end
        return info
      end
      
      def maxwell_auto_mxm_material_info(material)
        info = ""
        matname = material.name.downcase
        if (!RMG.windows_platform?)
          matname.gsub!(/\.mxm$/i, "") # OSX uses .mxm in sketchup names
        end
        material_name = RMG.encode64(matname)
        if (@auto_mxm_materials["#{material_name}"] != nil)
          filename = @auto_mxm_materials["#{material_name}"].first
          info << "<td class=\"auto\" #{context_menu(material, filename)}>"
          info << '<span class="mxm_file"'
          info << " onclick=\"mxm_file_onClick('#{RMG.encode64(filename)}')\">"
          info << RMG.escapeHTML(filename) << "</span></td>\n"
        else
          info << "<td #{context_menu(material, nil)}><span class=\"mxm_file\""
          info << " onclick=\"unassigned_mxm_file_onClick('#{RMG.encode64(material.name)}')\">"
          info << "no material assigned</span></td>\n"
        end
        return info
      end
     
      def maxwell_mxm_material_info(material)
        info = ""
        mxm = material.attribute_dictionary('MaxwellMXM')              
        filename = mxm['MXM']
        if (filename != nil)
          if (File.exist?(filename))
            info << "<td class=\"assigned\" #{context_menu(material, filename)}>"
            info << '<span class="mxm_file"'
            info << " onclick=\"mxm_file_onClick('#{RMG.encode64(filename)}')\">"
            info << RMG.escapeHTML(filename.gsub(/\\/, "/"))  << "</span></td>\n"
          else
            info << "<td class=\"missing\">#{RMG.escapeHTML(filename.gsub(/\\/, "/"))}</td>\n"
          end
        else
          info << "<td class=\"missing\" #{context_menu(material, nil)}><span class=\"mxm_file\""
          info << " onclick=\"unassigned_mxm_file_onClick('#{RMG.encode64(material.name)}')\">"
          info << "MXM: (file not found)</span></td>\n"
        end  
        return info
      end
      
      def maxwell_plastic_material_info(material)
        mxm = material.attribute_dictionary('MaxwellMXM')      
        info = "<td class=\"custom\" #{context_menu(material, nil)}>Plastic"
        if (mxm['Plastic_Roughness'] != nil)
          info << " (Shininess = #{RMG.escapeHTML(mxm['Plastic_Roughness'].to_s)}%)"
        end
        info << "</td>\n"
        return info
      end
      
      def maxwell_metal_material_info(material)
        mxm = material.attribute_dictionary('MaxwellMXM')        
        info = "<td class=\"custom\" #{context_menu(material, nil)}>Metal"
        if (mxm['Metal_Roughness'] != nil)
          info << " (Roughness = #{RMG.escapeHTML(mxm['Metal_Roughness'].to_s)}%)"
        end
        info << "</td>\n"
        return info
      end
      
      def maxwell_emitter_material_info(material)
        mxm = material.attribute_dictionary('MaxwellMXM')        
        info = "<td class=\"custom\" #{context_menu(material, nil)}>Emitter"
        if (mxm['Emitter_Watts'] != nil)
          info << " (Watts = #{RMG.escapeHTML(mxm['Emitter_Watts'].to_s)})"
        end
        if (mxm['Emitter_Efficiency'] != nil)
          info << " (Efficiency = #{RMG.escapeHTML(mxm['Emitter_Efficiency'].to_s)}%)"
        end
        info << "</td>\n"
        return info
      end      
      
    end #class

HTML_SUMMARY = <<EOF
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
    <title>MXM Reporter</title>
<style type="text/css">
<!--
body, h1, h2, h3, table, tr, th, td{
	margin: 0px;
	padding: 0px;
}
body {
	padding: 2%;
	font-size: 13px;
	font-family: Verdana, Geneva, sans-serif;
  color: #000;
	background-color: #FFF;
}
h1 {
 	font-family: "Times New Roman", Times, serif;
  font-size: 24px;
	color: #004F75;
	font-weight: bold;
	margin-bottom: 0.1em;
}
h2 {
	font-family: Verdana, Geneva, sans-serif;
	font-size: 14px;
	color: #666;
	font-style: italic;
	margin-bottom: 0.3em;
	font-weight: bold;
}
h3 {
  font-family: Verdana, Geneva, sans-serif;
  font-size: 13px;
  color: #000;
  font-weight: normal;
	text-decoration: underline;
}
.repeat {
	background-color: #F2C6C7;
}
-->
</style>
<script type="text/javascript">
    function select_material(material) {}
    function add_material(html) {}
    function remove_material(material_id) {}

    function button_onClick()
    {
        window.location = 'skp:onclick_refresh';
    }
</script>
</head>
<body>
insert_summary
<button onclick="button_onClick()">Continue</button>
</body>
</html>
EOF
    
    
HTML_REPORT = <<EOF
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
    <title>MXM Reporter</title>
<style type="text/css">
<!--
body , h1, h2, table, tr, th, td, ul, li, p {
	margin: 0px;
	padding: 0px;
}
body {
	padding: 2%;
	font-size: 13px;
	font-family: Verdana, Geneva, sans-serif;
	background-color: #FFF;
}
h1 {
	font-size: 32px;
	font-family: "Times New Roman", Times, serif;
	color: #004F75;
	font-weight: bold;
  line-height: 0.9em;
	margin-bottom: 0.2em;
}
h2 {
	font-family: Verdana, Geneva, sans-serif;
	font-size: 16px;
	color: #666;
	font-style: italic;
	margin-bottom: 1em;
	font-weight: bold;
}
table {
	border-collapse: collapse;
	width: 100%;
	background-color: #F7F8FB;
	cursor: default;  
}
tr.even {
	background-color: #EFF2F8;
}
th , td{
	border: 1px solid #666;
	padding-top: 0.5em;
	padding-right: 1em;
	padding-bottom: 0.5em;
	padding-left: 1em;
	color: #333;
	text-align: left;
}
td.selectable:hover {
	background-color: #FFC;
	cursor: pointer;
}
td.selected, td.selected:hover {
	background-color: #FC6;
}
th {
	background-color: #B7C9E3;
	font-weight: bold;
	text-align: left;
	font-size: 14px;
}

.auto {	background-color: #D9C6E3; }
.custom {	background-color: #E6D7BF; }
.assigned {	background-color: #C6E8C6; }
.missing { background-color: #F2C6C7; }
.unknown { background-color: #DDD; }

.mxm_file {
	color: #333;
	text-decoration: none;
	font-weight: normal;
}
.mxm_file:hover {
	color: #333;
	text-decoration: underline;
	cursor: pointer;
}

table.options {
	background-color: #FFF;
}
table.options td.left {
	border-top-style: none;
	border-right-style: none;
	border-bottom-style: none;
	border-left-style: none;
	text-align: left;
	padding: 0.5em;
	vertical-align: top;
}
table.options td.right {
	border-top-style: none;
	border-right-style: none;
	border-bottom-style: none;
	border-left-style: none;
	text-align: right;
	padding: 0.5em;
	vertical-align: top;
}
input#auto_mxm_path {
	font-family: "Courier New", Courier, monospace;
}
#refresh_button {
	font-size: 28px;
  color: #333;
}
ul {
	background-color: #D2DDEE;
	border: 1px solid #666;
	width: 170px;
	padding: 0.4em;
	list-style-type: none;
	color: #000;
	font-weight: normal;
	cursor: pointer;
	position: absolute;
	display: none;
}
li {
	padding-top: 0.2em;
	padding-right: 0.5em;
	padding-bottom: 0.2em;
	padding-left: 0.5em;
}
li:hover {
	background-color: #FFC;
	color: #000;
}
li.disabled {
	color: #999;
}
li.disabled:hover {
	color: #999;
	background-color: #D2DDEE;
	cursor: default;
}
-->
</style>

<script type="text/javascript">
    
    var mouse = {x:0, y:0};
    var context_material = "";
    var context_location = "";
    var context_clipboard = "";	
    
    function onLoad()
    {
        highlight_even_rows();
        document.onclick = hide_context_menu;
        document.onmousemove = mousemove;
    }
   
    function hide_context_menu()
    {
        var context_menu = document.getElementById('context_menu');
        context_menu.style.display = "none";
    }
    
    function show_context_menu(e, sketchup_material, mxm_location)
    {
        context_material = sketchup_material;
        context_location = mxm_location;
        
        var context_menu = document.getElementById('context_menu');
        update_context_menu_items();	
        context_menu.style.left = mouse.x + "px";
        context_menu.style.top = mouse.y + "px";
        context_menu.style.display = "block";
        adjust_context_menu_position(context_menu);		
        
        // prevent event from bubbling up & causing the browser context menu to appear.
        e.cancelBubble = true;
        if (e.stopPropagation) e.stopPropagation(); 
        if (e.preventDefault) e.preventDefault();
        return false;
    }
    
    function update_context_menu_items()
    {
      var copy_menu_item = document.getElementById('context_menu_copy_mxm');
      var paste_menu_item = document.getElementById('context_menu_paste_mxm');
      if (context_location)
      {
        copy_menu_item.className = "";
        copy_menu_item.onclick = context_menu_copy_mxm_location_onClick;
      }
      else
      {
        copy_menu_item.className = "disabled";
        copy_menu_item.onclick = null;
      }
      if (context_clipboard)
      {
        paste_menu_item.className = "";
        paste_menu_item.onclick = context_menu_paste_mxm_location_onClick;
      }
      else
      {
        paste_menu_item.className = "disabled";
        paste_menu_item.onclick = null;
      }	
    }
    
    // http://www.howtocreate.co.uk/tutorials/javascript/browserwindow
    function adjust_context_menu_position(context_menu)
    {
        var distance_to_right_edge = 0;
        var distance_to_bottom_edge = 0;
        
        scrolled = getScrollXY();
        win = getWindowDimensions();
        
        client_x = mouse.x - scrolled.x;
        client_y = mouse.y - scrolled.y;
        
        distance_to_right_edge = win.width - client_x;
        distance_to_bottom_edge = win.height - client_y;
        
        if (distance_to_right_edge < context_menu.offsetWidth)
            context_menu.style.left = mouse.x - context_menu.offsetWidth + "px";  
        if (distance_to_bottom_edge < context_menu.offsetHeight) 
            context_menu.style.top = mouse.y - context_menu.offsetHeight + "px";	
    }
    
    function mousemove(e)
    { 
        var scrolled = getScrollXY();
        if (e && e.clientX) // Mozilla
        {
            mouse.x = e.clientX + scrolled.x;
            mouse.y = e.clientY + scrolled.y;
        }
        else if (window.event && window.event.clientX)
        { 
            mouse.x = window.event.clientX + scrolled.x;
            mouse.y = window.event.clientY + scrolled.y;
        }
    }
    
    
    function getWindowDimensions()
    {
        var myWidth = 0, myHeight = 0;
        if (typeof(window.innerWidth) == 'number')
        {
            //Non-IE
            myWidth = window.innerWidth;
            myHeight = window.innerHeight;
        }
        else if (document.documentElement && 
                (document.documentElement.clientWidth || document.documentElement.clientHeight))
        {
            //IE 6+ in 'standards compliant mode'
            myWidth = document.documentElement.clientWidth;
            myHeight = document.documentElement.clientHeight;
        }
        else if (document.body && 
                (document.body.clientWidth || document.body.clientHeight))
        {
            //IE 4 compatible
            myWidth = document.body.clientWidth;
            myHeight = document.body.clientHeight;
        }
        return {width:myWidth, height:myHeight};
    }
    
    function getScrollXY()
    {
        var scrOfX = 0, scrOfY = 0;
        if (typeof(window.pageYOffset) == 'number')
        {
            //Netscape compliant
            scrOfY = window.pageYOffset;
            scrOfX = window.pageXOffset;
        } 
        else if (document.body && (document.body.scrollLeft || document.body.scrollTop ))
        {
            //DOM compliant
            scrOfY = document.body.scrollTop;
            scrOfX = document.body.scrollLeft;
        }
        else if (document.documentElement && 
                (document.documentElement.scrollLeft || document.documentElement.scrollTop)) 
        {
            //IE6 standards compliant mode
            scrOfY = document.documentElement.scrollTop;
            scrOfX = document.documentElement.scrollLeft;
        }
        return {x:scrOfX, y:scrOfY};
    }   
    
    
    
    function highlight_even_rows()
    {
        var tbody = document.getElementById('report').tBodies[0];
        for (i = 0; i < tbody.rows.length; i++)
        {
            if ((i+1) % 2 == 0) tbody.rows[i].className="even";
            else tbody.rows[i].className="";
        }
    }    
    
    function unselect_materials()
    {
        var elems = document.getElementsByTagName('td');
        for (i = 0; i < elems.length; i++)
        {
            if (elems[i].className.match("selected"))
            {
                elems[i].className = "selectable";
            }
          
        }
    }
    
    function select_material(material)
    {
        unselect_materials();
        elem = document.getElementById(material);
        if (elem != null)
        {
            elem.className = "selectable selected";
        }
    }
    
    function add_material(material_id, html)
    {
        var temp = document.createElement('div');
        temp.innerHTML = '<table><tbody><tr>' + html + '</tr></tbody></table>'
        var newrow = temp.getElementsByTagName('tr')[0];
        var table = document.getElementById('report')
        var tbody = table.tBodies[0];
        var oldrow = tbody.insertRow(tbody.rows.length);
        tbody.replaceChild(newrow, oldrow);
        newrow.id = material_id;
        temp = null;
        highlight_even_rows();
    }
    
    function update_material(material_id, html)
    {
        var oldrow = document.getElementById(material_id);
        if (oldrow == null) return;
        oldrow.id = "";
        var temp = document.createElement('div');
        temp.innerHTML = '<table><tbody><tr>' + html + '</tr></tbody></table>';
        var newrow = temp.getElementsByTagName('tr')[0];
        var table = document.getElementById('report');
        var tbody = table.tBodies[0];
        tbody.replaceChild(newrow, oldrow);
        newrow.id = material_id;
        newrow.className = oldrow.className;
        temp = null;
    }  
    
    function remove_material(material_id)
    {
        var td = document.getElementById(material_id)
        if (td != null)
        {
            var table = document.getElementById('report')
            var tbody = table.tBodies[0];
            var row = td.parentNode;
            tbody.deleteRow(row.rowIndex);
       			highlight_even_rows();
        }	
    }
    
    function refresh_button_onClick()
    {
        window.location='skp:onclick_refresh';	
		}
    
    function auto_mxm_toggle_onChange()
    {
        var combobox = document.getElementById('auto_mxm_toggle');
        var selected = combobox.options[combobox.selectedIndex];
        if(selected.value == "enabled")
        {
            document.getElementById('auto_mxm_path').disabled = false;
            document.getElementById('scan_button').disabled = false;
        }
        else
        {
            document.getElementById('auto_mxm_path').disabled = true;
            document.getElementById('scan_button').disabled = true;
        }
        window.location='skp:onchange_auto_mxm_toggle';
    }
    
    function scan_button_onClick()
    {
        window.location='skp:onclick_scan_auto_mxm_path';
    }
    
	  function sketchup_material_onClick(str)
    {
	      window.location = 'skp:onclick_sketchup_material@' + str;
    }
    
    function mxm_file_onClick(str)
    {
        window.location = 'skp:onclick_mxm_file@' + str;
    }
    function unassigned_mxm_file_onClick(str)
    {
        window.location = 'skp:onclick_unassigned_mxm_file@' + str;
    }
    
    function context_menu_choose_mxm_file_onClick()
    {
        if (context_location)
          var params = context_material + '&' + context_location;
        else
          var params = context_material;
        window.location = 'skp:onclick_assigned_mxm_file@' + params;
    }
    function context_menu_copy_mxm_location_onClick()
    {
        context_clipboard = context_location;
    }
    function context_menu_paste_mxm_location_onClick()
    {
        var params = context_material + '&' + context_clipboard;
        window.location = 'skp:onclick_paste_mxm_location@' + params;
    }    
    
    function clear_selected_onClick()
    {
        window.location = 'skp:onclick_remove_mxm_dictionary';
    } 
    function clear_all_onClick()
    {
        window.location = 'skp:onclick_remove_all_mxm_dictionaries';
    } 
</script>
</head>
<body onload="onLoad()">

<table class="options">
<tr>
    <td class="left">
        <h1>MXM Reporter</h1>
        <h2>Version 0.4b</h2>
    </td>
    <td class="right">
        <button id="refresh_button" onclick="refresh_button_onClick()">Refresh</button>
    </td>
</tr>
</table>

<table class="options">
<tr>
    <td class="left" style="vertical-align:baseline">
        Auto MXM: 
        <select id="auto_mxm_toggle" onchange="auto_mxm_toggle_onChange()">
            <option value="disabled">Disabled</option>
            <option value="enabled"INSERT_AUTOMXM_SELECTED>Enabled</option>
        </select>
    </td>
    <td class="right" style="vertical-align:baseline">
        Auto MXM Path:
        <input type="text" id="auto_mxm_path"INSERT_AUTOMXM_PATH_DISABLED value="INSERT_AUTOMXM_LOCATION" />
        <button id="scan_button"INSERT_SCAN_BUTTON_DISABLED onclick="scan_button_onClick()">Scan</button>
    </td>
</tr>
</table>

<table id="report" onselectstart="return false">
<tr><th>SKETCHUP</th><th>MAXWELL</th></tr>
INSERT_REPORT
</table>
<br />
<button onclick="clear_selected_onClick()">Clear Selected</button>&nbsp;
<button onclick="clear_all_onClick()">Clear All</button>


<ul id="context_menu">
<li onclick="context_menu_choose_mxm_file_onClick()">Choose MXM file...</li>
<li id="context_menu_copy_mxm">Copy MXM location</li>
<li id="context_menu_paste_mxm">Paste MXM location</li>
</ul>


</body>
</html>
EOF




end # MXM_REPORTER module
end # RMG module
